/*
  Class pr is defined here. All PR classes are derived from it and
  thus have the ability to initialize and terminate pr library
  on their own without explicit user interference.
  Global error routines are also defined here.
*/

#ifndef _PR_GENERAL_H
#define _PR_GENERAL_H


// Error handling
#ifdef WIN32
#define PR_ERRNO (GetLastError())
#else
#define PR_ERRNO errno
#endif // WIN32

/*
  Return error string for system error codes. Portable-safe.
  Returned string should not be changed by user in any way.
*/
char* pr_strerror(unsigned long);


// pr abstract class
class pr{
protected:
// Constructor is protected so that user can not create object of class pr
	pr();
	~pr();
/*
  Setup any PR internal data structures. This MUST be the first
  function called for any PR library call.
*/
	void pr_init();

// Do any cleanup functions when library is not needed anymore
	void pr_terminate();

// Counter of modules that are connected to the library
	static int count;
};

// Function that returns full path to the module being executed
void GetFullPath(char*, char*, unsigned int);

#endif // _PR_GENERAL_H