#ifndef _PROTOCOL_H
#define _PROTOCOL_H

#include "pr_socket.h"
#include "handler.h"
#include "rrparser.h"
#include "rrdata.h"

class Protocol{
public:
	~Protocol();

	// reads socket and forms RRData structure or returns false in case of error
	bool Parse(RRData& in);
	virtual bool Generate(RRData& out) = 0;
	virtual bool Handle(RRData::t_params = RRData::t_params(0, 0)) = 0;
	
	// return true if underlying socket is online
	bool Online(void);

	// return true if last Parse() execution returned partial execution
	bool PartialExecution(void);
	
	RRParser::rrp_error_t getParserError(void);
	
protected:
	Protocol(stdSocket* sock, Handler* handler);
	int calculateSize(RRData& a);

	CBuffer* buffer_;
	Handler* handler_;
	stdSocket* socket_;
	RRParser* parser_;

	RRData in;
	RRData out;

private:
	Protocol(){}
};

class ClientProtocol: public Protocol{
public:
	ClientProtocol(stdSocket* sock, Handler* handler);
	~ClientProtocol();
	// takes RRData structure, forms plain request's text array from it and sends it through socket
	bool Generate(RRData& out);
	// invokes Parse() and stores result
	virtual bool Handle(RRData::t_params = RRData::t_params(0, 0));
};

class ServerProtocol: public Protocol{
public:
	ServerProtocol(stdSocket* sock, Handler* handler);
	~ServerProtocol();
	// takes RRData structure, forms plain response's text array from it and sends it through socket
	bool Generate(RRData& out);
	// invokes Parser(), then appropriate handler and after that Generate()
	virtual bool Handle(RRData::t_params = RRData::t_params(0, 0));
};


#endif // _PROTOCOL_H