#ifndef _RRPARSER_H
#define _RRPARSER_H

#define MAX_LINE_SIZE 1024
#define MAX_HEADERS_COUNT 1024

#include <iostream>
#include <string>
#include <map>
#include <vector>
#include <algorithm>
#include "buffer.h"
#include "rrdata.h"
#include "handler.h"



///////////////////////////////////////////////////////////////////////////////////////////
// ==================================== class RRParser ====================================

class RRParser
{
public:
	RRParser(CBuffer& buffer, Handler& handler);
	virtual ~RRParser();

	bool Parse(RRData* pRRData);
	enum rrp_error_t { rrp_no_error, rrp_request_error, rrp_part_command,
						rrp_part_header, rrp_part_body, rrp_buffer_error };
	rrp_error_t Error() const { return error_; }
	
protected:
	CBuffer& buffer_;
	rrp_error_t error_;
	Handler& handler_;

	virtual bool ParseString(RRData* pRRData) = 0;
	bool ParseHeaders(RRData* pRRData);
	bool ParseBody(RRData* pRRData);

	static bool ParseProtocolVersion(const std::string& line, RRData* pRRData);
	static bool ParseHeader(std::string line, RRData::t_header& header);

	static std::vector< std::string > Split(const std::string& line);
	static std::string SplitWord(std::string& line);
	static bool Space(char ch) { if (ch < 0) ch = 0; return isspace(ch) ? true : false; }
	static bool NotSpace(char ch) { if (ch < 0) ch = 0; return isspace(ch) ? false : true; }
};

///////////////////////////////////////////////////////////////////////////////////////////
// ================================= class RRServerParser =================================

class RRServerParser : public RRParser
{
public:
	RRServerParser(CBuffer& buffer, Handler& handler): RRParser(buffer, handler) {}
protected:
	virtual bool ParseString(RRData* pRRData);
};

///////////////////////////////////////////////////////////////////////////////////////////
// ================================= class RRClientParser =================================

class RRClientParser : public RRParser
{
public:
	RRClientParser(CBuffer& buffer, Handler& handler): RRParser(buffer, handler) {}
protected:
	virtual bool ParseString(RRData* pRRData);
};

// ========================================================================================
///////////////////////////////////////////////////////////////////////////////////////////

#endif // _RRPARSER_H