//CompilerService.cpp - impliementation of CCompilerService class
#include "CompilerService.h"
#include "Interface.h"
#include "Compiler.h"
#include <string.h>
#include <stdio.h>
#include "Define.h"
#include <windows.h>

CCompilerService::CCompilerService(CFolder* _folder,CInterface* _screen):CFViser(_folder,_screen)
{
	compilers=NULL;
	compilers_string=NULL;
};

int CCompilerService::init(char* _buff)
{
	char _s1[CMP_MAX_LENGTH];
	char _s2[CMP_MAX_LENGTH];
	char _s3[CMP_MAX_LENGTH];
	char _s4[CMP_MAX_LENGTH];
	char _msg[100];
	unsigned int _timeout;
	char _oldend='\n';
	int _res=AER_NO_ERROR;
	int cs_length=0;

	char* _start=_buff;

	while(_start!=NULL)
	{
		char* _begin=strstr(_start,COMPILER_CHAPTER);
		if(_begin==NULL)
			break;
		char* _end=strstr(_begin,DOUBLE_NEW_LINE);
		if(_end)
		{
			_oldend=*_end;
			*_end='\0';
		}
		
		SCmpNode* _node=new SCmpNode;				// create new node

		try
		{
			if(get_string(_begin,CMP_NAME,_s1)==false)			// extract name
				throw IER_WRONG_INI_FILE;
			_node->name=new char[strlen(_s1)+1];
			strcpy(_node->name,_s1);
			cs_length+=(int)strlen(_s1)+1;

			if(get_string(_begin,CMP_FILE,_s1)==false)			// extract file path
				throw IER_WRONG_INI_FILE;
			if(get_string(_begin,CMP_OPTIONS,_s2)==false)		// extract command line
				throw IER_WRONG_INI_FILE;
			if(get_string(_begin,CMP_WORKFOLDER,_s3)==false)		// extract work folder
				throw IER_WRONG_INI_FILE;
			if(get_string(_begin,CMP_EXTENSION,_s4)==false)		// extract extension
				throw IER_WRONG_INI_FILE;
			if(get_uint(_begin,CMP_TIMEOUT,&_timeout)==false)	// extract time
				throw IER_WRONG_INI_FILE;
		}
		catch(int err)
		{
			delete _node;
			this->delete_compilers();
			_res=err;
			if(_end)
				*_end=_oldend;
			break;
		};

		sprintf(_msg,"\n\nLoad compiler \"%s\" :",_node->name);
		screen->print(_msg);

		_node->compiler=new CCompiler(folder,screen,_s1,_s2,_s3,_s4,_timeout);
		_node->next=compilers;
		compilers=_node;


		screen->print("\ncheck compiler file name ... ");
		
		if(folder->file_exists(_s1)==false)
		{
			screen->print("error");
			screen->print("\ncompiler file does not exist. check ini file.");
			screen->print("\nI will remove it from compilers list.");
			compilers=_node->next;
			cs_length-=(int)strlen(_node->name)+1;
			delete _node->compiler;
			delete _node->name;
			delete _node;
		}
		else
		{
			screen->print("done");
		}

		if(_end)
			*_end=_oldend;
		_start=_end;
	}

	if(_res!=AER_NO_ERROR)
		return _res;

	compilers_string=new char[cs_length+1];
	compilers_string[0]='\0';
	SCmpNode* _p=compilers;
	int _i=0;
	while(_p!=NULL)
	{
		strcat(compilers_string,_p->name);
		if(_p->next!=NULL)
			strcat(compilers_string,";");
		_p=_p->next;
		_i++;
	};

	if(_i==0)
		return IER_NO_COMPILERS;

	sprintf(_s1,"\n\n%u compilator(s) was loaded succesfuly",_i);
	screen->print(_s1);
	sprintf(_s1,"\ncreated compilers string: '%s'",compilers_string);
	screen->print(_s1);

	return _res;
};

int CCompilerService::compile(const char* _source,const char* _cmpname)
{
	SCmpNode* _p=compilers;
	while(_p!=NULL)
	{
		if(strcmp(_p->name,_cmpname)==0) break;
		_p=_p->next;
	};
	if(_p==NULL)
	{
		return AER_WRONG_COMPILER;
	};
	return _p->compiler->compile(_source);
};

const char* CCompilerService::get_compilers(void)
{
	return (const char*)compilers_string;
};

void CCompilerService::delete_compilers(void)
{
	while(compilers!=NULL)
	{
		SCmpNode* _temp=compilers;
		compilers=_temp->next;
		delete _temp->compiler;
		delete _temp->name;
		delete _temp;
	};
}

CCompilerService::~CCompilerService()
{
	this->delete_compilers();
	delete[] compilers_string;
}