// Console.cpp - implementation file for CConsole class
#include "Console.h"
#include "Define.h"
#include <windows.h>
#include <string.h>
#include <stdio.h>
#include "Define.h"
#include "Tasks.h"

CConsole::CConsole()
{
	tasks_number=0;

	std_out=GetStdHandle(STD_OUTPUT_HANDLE);
	std_in=GetStdHandle(STD_INPUT_HANDLE);
	SetConsoleTitle("Tester");

	// console size
	x_size=80;
	y_size=50;

	show_border();

	new_line.X=0;
	new_line.Y=(SHORT)y_size-1;

	cursor_pos=new_line;

	// apply
	SMALL_RECT _rect;
	_rect.Bottom=(SHORT)y_size-1;
	_rect.Top=0;
	_rect.Left=0;
	_rect.Right=(SHORT)x_size-1;

	COORD _coord;
	_coord.X=(SHORT)x_size;
	_coord.Y=(SHORT)y_size;

	SetConsoleWindowInfo(std_out,true,&_rect);
	SetConsoleScreenBufferSize(std_out,_coord);

	SetConsoleCursorPosition(std_out,new_line);
};

void CConsole::update_tasks(void)
{
	if(tasks_number!=tasks->get_quantity())
		redraw_border(tasks->get_quantity());
    update_border();
};

void CConsole::redraw_border(unsigned int _task_number)
{
	hide_border();
	tasks_number=_task_number;
	show_border();
};

void CConsole::hide_border(void)
{
	char _buff[80];
	unsigned int i;
	for(i=0;i<80;i++)
		_buff[i]=' ';
	for(i=0;i<=tasks_rect.Bottom;i++)
		printxy(0,i,_buff,80);
};

void CConsole::show_border(void)
{
	char _buff[80];
	unsigned int i;

	scroll_rect.Top=6+2*tasks_number;
	scroll_rect.Left=0;
	scroll_rect.Right=(SHORT)x_size-1;
	scroll_rect.Bottom=(SHORT)y_size-1;

	new_scroll_pos.X=0;
	new_scroll_pos.Y=scroll_rect.Top-1;

	tasks_rect.Top=0;
	tasks_rect.Left=0;
	tasks_rect.Right=(SHORT)x_size-1;
	tasks_rect.Bottom=new_scroll_pos.Y-1;
    
	char_fill.Char.AsciiChar=' ';
	char_fill.Attributes=0;

	table_hline(_buff,'\xCD','\xD5','\xB8','\xD1');
	printxy(0,0,_buff,80);

	table_hline(_buff,' ','\xB3','\xB3','\xB3');
	printxy(0,1,_buff,80);

	printxy(10,1,"Tasks:",6);
	printxy(HPOS_TASK+1,1,"Received",8);
	printxy(HPOS_RECV+1,1,"Accepted",8);
	printxy(HPOS_ACTD+1,1,"Refused",7);	

	table_hline(_buff,'\xC4','\xC3','\xB4','\xC5');
	printxy(0,2,_buff,80);

	for(i=3;i<tasks_rect.Bottom;i++)
	{
		table_hline(_buff,' ','\xB3','\xB3','\xB3');
		printxy(0,i,_buff,80);
	};

	table_hline(_buff,'\xCD','\xD4','\xBE','\xCF');
	printxy(0,tasks_rect.Bottom,_buff,80);
};

void CConsole::update_border(void)
{
	unsigned int i;
	char _buff[80];

	for(i=0;i<tasks_number;i++)
	{
		sprintf(_buff,"%2u. %s",i+1,(*tasks)[i]->get_name());
		printxy(2,4+2*i,_buff,strlen(_buff));
		
		sprintf(_buff,"%3u",(*tasks)[i]->get_total());
		printxy(HPOS_TASK+3,4+2*i,_buff,strlen(_buff));

		sprintf(_buff,"%3u",(*tasks)[i]->get_accept());
		printxy(HPOS_RECV+3,4+2*i,_buff,strlen(_buff));

		sprintf(_buff,"%3u",(*tasks)[i]->get_refuse());
		printxy(HPOS_ACTD+3,4+2*i,_buff,strlen(_buff));
	};
};

void CConsole::table_hline(char* _buff,char _symb,char _first,char _last,char _delim)
{
	int i;
	_buff[0]=_first;
	for(i=1;i<x_size-1;i++)
	{
		if(i==HPOS_TASK || i==HPOS_RECV || i==HPOS_ACTD)
			_buff[i]=_delim;
		else
			_buff[i]=_symb;
	}
	_buff[x_size-1]=_last;
};

void CConsole::printxy(unsigned int _x,unsigned int _y,const char* _text,unsigned int _n)
{
	COORD _coord,_old;
	
	CONSOLE_SCREEN_BUFFER_INFO _info;
	GetConsoleScreenBufferInfo(std_out,&_info);
	_old=_info.dwCursorPosition;

	_coord.X=(SHORT)_x;
	_coord.Y=(SHORT)_y;

	SetConsoleCursorPosition(std_out,_coord);
	WriteConsole(std_out,(const void*)_text,(DWORD)_n,&printed,NULL);
	SetConsoleCursorPosition(std_out,_old);
}

int CConsole::init(char* _buff)
{
	return this->apply();
};

int CConsole::apply(void)
{
	return AER_NO_ERROR;
};

void CConsole::print(const char* _text)
{
	int i;
	char c;
	void* p=&c;

	for(i=0;_text[i]!='\0';i++)
	{
		if((c=_text[i])>=32&&cursor_pos.X<x_size-1)
		{
			cursor_pos.X++;
			WriteConsole(std_out,p,1,&printed,NULL);
		}
		if(_text[i]=='\n')
		{
			ScrollConsoleScreenBuffer(std_out,&scroll_rect,NULL,new_scroll_pos,&char_fill);
			SetConsoleCursorPosition(std_out,new_line);
			cursor_pos=new_line;
		}
	};
};

CConsole::~CConsole()
{
	CloseHandle(std_out);
	CloseHandle(std_in);
};