// Folder.cpp - implementation of folder storage class CFolder
#include <string.h>
#include <stdio.h>
#include <windows.h>
#include "Define.h"
#include "Folder.h"

CFolder::CFolder(CInterface* _screen)
{
	screen=_screen;
	work_folder=NULL;
	source_folder=NULL;
	test_folder=NULL;
};

const char* CFolder::get_wf(void)
{
	return (const char*)work_folder;
};

const char* CFolder::get_sf(void)
{
	return (const char*)source_folder;
};

const char* CFolder::get_tf(void)
{
	return (const char*)test_folder;
};

int CFolder::store_file(const char* _filename,const char* _buff,unsigned long _size)
{
	HANDLE _file=CreateFile(_filename,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
	if(_file==INVALID_HANDLE_VALUE) return RER_CREATE_FILE;
	unsigned long _writen;
	WriteFile(_file,_buff,_size,&_writen,NULL);
	if(_writen!=_size)
	{
		CloseHandle(_file);
		return RER_WRITE_FILE;
	};
	CloseHandle(_file);
	return AER_NO_ERROR;
};

int CFolder::make_folder(const char* _path)
{
	return setup_folder(_path,false);
};

int CFolder::copy_file(const char* _to,const char* _from)
{
	if(CopyFile(_from,_to,FALSE)==FALSE)
		return RER_COPY_FILE;
	return AER_NO_ERROR;
};

int CFolder::delete_file(const char* _file)
{
	if(DeleteFile(_file)==FALSE)
		return RER_DELETE_FILE;
	return AER_NO_ERROR;
}

int CFolder::init(char* _buff)
{
	char _folder[FLD_MAX_LENGTH];
	char _msg[100];
	char _old=' ';

	char* _begin=strstr(_buff,FOLDER_CHAPTER);
	if(_begin==NULL)
		return IER_NO_INFORMATION;
	char* _end=strstr(_begin,DOUBLE_NEW_LINE);
	if(_end)
	{
		_old=*_end;
		*_end='\0';
	};

	try
	{
		screen->print("\n\nLoad folder settings...");
		screen->print("\nset source folder to ");
		if(get_string(_buff,FLD_SOURCE,_folder)==0)
			throw IER_WRONG_INI_FILE;
		source_folder=new char[strlen(_folder)+1];
		strcpy(source_folder,_folder);
		sprintf(_msg,"\"%s\"",source_folder);
		screen->print(_msg);

		screen->print("\nset work folder to ");
		if(get_string(_buff,FLD_WORK,_folder)==0)
			throw IER_WRONG_INI_FILE;
		work_folder=new char[strlen(_folder)+1];
		strcpy(work_folder,_folder);
		sprintf(_msg,"\"%s\"",work_folder);
		screen->print(_msg);

		screen->print("\nset test folder to ");
		if(get_string(_buff,FLD_TEST,_folder)==0)
			throw IER_WRONG_INI_FILE;
		test_folder=new char[strlen(_folder)+1];
		strcpy(test_folder,_folder);
		sprintf(_msg,"\"%s\"",test_folder);
		screen->print(_msg);
	}
	catch(int _error)
	{
		delete source_folder;
		delete work_folder;
		delete test_folder;
		work_folder=NULL;
		source_folder=NULL;
		test_folder=NULL;
		if(_end) *_end=_old;
		return _error;
	};

	if(_end)
		*_end=_old;

	return check_folders();
};

int CFolder::check_folders(void)
{
	int _res;
	screen->print("\n\nCheck loaded settings...");
	
	screen->print("\ncheck source folder ... ");
	_res=setup_folder(source_folder,true);
	if(_res!=AER_NO_ERROR)
		return _res;

	screen->print("\ncheck work folder ... ");
	_res=setup_folder(work_folder,true);
	if(_res!=AER_NO_ERROR)
		return _res;

	screen->print("\ncheck test files folder ... ");
	_res=setup_folder(test_folder,true);
	if(_res!=AER_NO_ERROR)
		return _res;

	return AER_NO_ERROR;
};

int CFolder::clear_wf(void)
{
	return setup_folder(work_folder,true);
};

int CFolder::setup_folder(const char* _folder_path,bool _msg)
{
	char _path[FLD_MAX_LENGTH];
	bool _no_files=false;
	int _result=AER_NO_ERROR;
	strcpy(_path,_folder_path);
	strcat(_path,"*.*");

	WIN32_FIND_DATA _file_data;
	HANDLE _find;

	_find=FindFirstFile(_path,&_file_data);
	if(_find==INVALID_HANDLE_VALUE)
	{
		if(_msg)
			screen->print("not exist");
		_result=create_dir(_folder_path);
		if(_msg)
		{
			if(_result==AER_NO_ERROR)
				screen->print(" (creating done)");
			else
				screen->print(" (creation fail)");
		}
	}
	else
	{
		if(_msg)
			screen->print("exist");
		while(_no_files==false)
		{
			if(strcmp(_file_data.cFileName,".")!=0 && strcmp(_file_data.cFileName,"..")!=0)
			{
				strcpy(_path,_folder_path);
				strcat(_path,_file_data.cFileName);
				SetFileAttributes(_path,0);

				if((_file_data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)!=0)
				{
					strcat(_path,"\\");
					if(_result==AER_NO_ERROR)
						_result=setup_folder(_path,false);
					else
						setup_folder(_path,false);
					if(RemoveDirectory(_path)==0 && _result==AER_NO_ERROR)
						_result=IER_CANT_REMOVE_DIR;
				}
				else
				{
					if(DeleteFile(_path)==0 && _result==AER_NO_ERROR)
						_result=IER_CANT_REMOVE_FILE;
				}
			}
			if (FindNextFile(_find,&_file_data)==0)
			{
				if (GetLastError()==ERROR_NO_MORE_FILES)
					_no_files=true;
				else
					_result=IER_UNKNOWN_ERROR;
			}
		}
		if(_msg)
		{
			if(_result==AER_NO_ERROR)
				screen->print(" (clearing done)");
			else
				screen->print(" (clearing fail)");
		}
	}
	FindClose(_find);

	return _result;
};

bool CFolder::file_exists(const char* _file)
{
	HANDLE _hfile=CreateFile(_file,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
	if(_hfile==INVALID_HANDLE_VALUE)
		return false;
	CloseHandle(_hfile);
	return true;
};

int CFolder::rename_file(const char* _to, const char* _from)
{
	if(MoveFile(_from,_to)==FALSE)
		return RER_MOVE_FILE;
	return AER_NO_ERROR;
};

char* CFolder::create_path(const char* _folder,const char* _file,const char* _ext)
{
	char* _path=new char[strlen(_folder)+strlen(_file)+10];
	if(_ext!=NULL)
        sprintf(_path,"%s%s%s",_folder,_file,_ext);
	else
		sprintf(_path,"%s%s",_folder,_file);
	return _path;
};

void CFolder::delete_path(char* _path)
{
	delete[] _path;
	_path=NULL;
};

int CFolder::delete_dir(const char* _path)
{
	int _res;
	try
	{
		if((_res=this->make_folder(_path))!=AER_NO_ERROR)
			throw _res;
		if((RemoveDirectory(_path)==0))
			throw IER_CANT_REMOVE_DIR;
	}
	catch(int _error)
	{
		_res=_error;
	};
	return _res;
};

int CFolder::create_dir(const char* _path)
{
	int _result=AER_NO_ERROR,_i;
	char _temp[1000];

	for(_i=0;_path[_i]!='\0';_i++)
	{
		_temp[_i]=_path[_i];
		if(_path[_i]=='\\')
		{
			_temp[_i+1]='\0';
			if(CreateDirectory(_temp,NULL)==0)
				_result=IER_CANT_CREATE_DIR;
			else
				_result=AER_NO_ERROR;
		}
	};
	return _result;
};

CFolder::~CFolder()
{
	this->delete_dir(work_folder);
	delete work_folder;
	this->delete_dir(source_folder);
	delete source_folder;
	this->delete_dir(test_folder);
	delete test_folder;
};