// Header.cpp - headers for ConTester

#include "Headers.h"
#include "RRData.h"
#include "Define.h"
#include "TestInfo.h"
#include "TestService.h"
#include <string.h>
#include <windows.h>

using namespace std;

namespace handlers
{

bool StringPrefix(const string& _str,const unsigned long _pos,const char* _string)
{
	long i;
	for(i=0;_str[_pos+i]==_string[i] && _string[i]!='\0';i++);
	return (_string[i]=='\0');
};

void FindSTR(const string& _str,unsigned long& _pos,const char* _string)
{
	_pos=_str.find(_string,_pos);
};

void SkipSTR(const string& _str,unsigned long& _pos,const char* _string)
{
	unsigned long _next=_str.find(_string,_pos);
	if(_next!=_pos) return;
	_pos+=strlen(_string);
	SkipSTR(_str,_pos,_string);
};

void SkipStringPrefix(const string& _str,unsigned long& _pos, const char* _string)
{
	long i;
	for(i=0;_str[_pos+i]==_string[i] && _string[i]!='\0';i++);
	_pos+=i;
};

void ReadSkipNumber(const string& _str,unsigned long& _pos,long& _number)
{
	char* _numb;
	long i;
	for(i=0;isdigit(_str[_pos+i]);i++);
	_numb=new char[i+1];
	for(i=0;isdigit(_str[_pos+i]);i++)
		_numb[i]=_str[_pos+i];
	_numb[i]='\0';
	_pos+=i;
	_number=atol(_numb);
	delete[] _numb;
};

void ReadIdValue(const string& _str,unsigned long& _pos,const char* _id,long& _number)
{
	SkipStringPrefix(_str,_pos,_id);
	ReadSkipNumber(_str,_pos,_number);
	SkipSTR(_str,_pos,HDR_SPACE);
};

void SkipPrefix(const string& _str,unsigned long& _pos,const char* _prefix)
{
	SkipStringPrefix(_str,_pos,HDR_TEST_ID);
	SkipSTR(_str,_pos,HDR_SPACE);
};

void ReadBody(const string& _str,unsigned long& _pos,const unsigned long _length,char* _buff)
{
	long i;
	for(i=0;i<_length;i++)
	{
		if(StringPrefix(_str,_pos,HDR_CRLF))
			SkipSTR(_str,_pos,HDR_CRLF);
		_buff[i++]=_str[_pos++];
	};
	FindSTR(_str,_pos,HDR_CRLF);
};

void ReadTestTaskBlock(const string& _str,unsigned long& _pos,STestInfo& _test_info,STestInfo::SBody& _body)
{
	long _length;
	ReadIdValue(_str,_pos,"Task:",_test_info.task_number);
	ReadIdValue(_str,_pos,"Test:",_test_info.test_number);
	ReadIdValue(_str,_pos,"Length:",_length);
	
	_body.body=new char[_length];
	_body.size=_length;
	_body.exist=true;
	ReadBody(_str,_pos,_length,_body.body);
};

void ReadTaskBlock(const string& _str,unsigned long& _pos,STestInfo& _test_info,STestInfo::SBody& _body)
{
	long _length;
	ReadIdValue(_str,_pos,"Task:",_test_info.task_number);
	ReadIdValue(_str,_pos,"Length:",_length);
	
	_body.body=new char[_length];
	_body.size=_length;
	_body.exist=true;
	ReadBody(_str,_pos,_length,_body.body);
}

void ReadBlock(const string& _str,unsigned long& _pos)
{
	long _length,i;
	STestInfo _test_info;
	ZeroMemory(&_test_info,sizeof(_test_info));
	SkipSTR(_str,_pos,HDR_CRLF);
	if(StringPrefix(_str,_pos,HDR_TEST_ID))
	{
		SkipPrefix(_str,_pos,HDR_TEST_ID);
		ReadTestTaskBlock(_str,_pos,_test_info,_test_info.input);
		return;
	};
	if(StringPrefix(_str,_pos,HDR_OUTPUT_ID))
	{
		SkipPrefix(_str,_pos,HDR_OUTPUT_ID);
		ReadTestTaskBlock(_str,_pos,_test_info,_test_info.output);
		return;
	};
	if(StringPrefix(_str,_pos,HDR_FILTER_ID))
	{
		SkipPrefix(_str,_pos,HDR_FILTER_ID);
		ReadTaskBlock(_str,_pos,_test_info,_test_info.filter);
		return;
	};
	if(StringPrefix(_str,_pos,HDR_GRAMMAR_ID))
	{
		SkipPrefix(_str,_pos,HDR_GRAMMAR_ID);
		ReadTaskBlock(_str,_pos,_test_info,_test_info.grammar);
		return;
	};
	if(StringPrefix(_str,_pos,HDR_SOLUTION_ID))
	{
		SkipPrefix(_str,_pos,HDR_SOLUTION_ID);
		ReadTaskBlock(_str,_pos,_test_info,_test_info.solution);
		return;
	};
};

void tester203(RRData& _in,RRData&)
{
	unsigned long i,j,k;
	unsigned int _task_number;
	int* _tasks;
	unsigned int _blocks_number;
	unsigned int _content_length;
	
	string _str;
	basic_string<char>::iterator _str_iterator;

	if(_in.headers.find("Tasks-Number")!=_in.headers.end())
	{
		_str=_in.headers["Tasks-Number"];
		_task_number=atoi(_str.c_str());
		
		_tasks=new int[_task_number];
		for(i=0;i<_task_number;i++)
			_tasks[i]=i+1;
	};
	if(_in.headers.find("Tasks")!=_in.headers.end())
	{
		_str=_in.headers["Tasks"];
		_task_number=0;
		for(i=0;i<_str.length();i++)
			if(_str[i]=';')
				_task_number++;
		_task_number+=1;
		_tasks=new int[_task_number];
		char _tmp[10];
		char _c;
		j=k=0;
		for(i=0;i<_str.length();i++)
		{
			_c=_str[i];
			if(_c<='9'&&_c>='0')
				_tmp[j++]=_c;
			if(_c==';')
			{
				_tmp[j]='\0';
				j=0;
				_tasks[k++]=atoi(_tmp);
			};
		};
		_tmp[j]='\0';
		_tasks[k++]=atoi(_tmp);
	};

	_str=_in.headers["Blocks-Number"];
	_blocks_number=atoi(_str.c_str());

	_str=_in.headers["Content-Length"];
	_content_length=atoi(_str.c_str());

	// read tests
	unsigned long _pos=0;
	for(i=0;i<_blocks_number;i++)
	{
		ReadBlock(_str,_pos);
	};
};

};