// Problem.cpp - implementation file for CProblem class
#include "Problem.h"
#include "Define.h"
#include "TestInfo.h"
#include "TaskInfo.h"
#include "MyFunc.h"
#include "FViser.h"
#include "CompilerService.h"
#include "FilterService.h"
#include <string.h>
#include <stdio.h>

CProblem::STestNode::STestNode(CTest* _test,int _number)
{
	test=_test;
	number=_number;
	next=NULL;
};

void CProblem::STestNode::add_next(STestNode* _add)
{
	if(next==NULL)
	{
		next=_add;
	}
	else
	{
		if(_add->number>next->number)
			next->add_next(_add);
		else
		{
			_add->next=next;
			next=_add;
		};
	};
};

CProblem::STestNode* CProblem::STestNode::find_number(int _number)
{
	if(number==_number)
		return this;
	if(next!=NULL)
		return next->find_number(_number);
	else
		return NULL;
};

CProblem::STestNode::~STestNode()
{
	delete test;
	delete next;
};

CProblem::CProblem(const char* _folder,CFolder* _cfolder,CInterface* _screen,CCompilerService* _compiler):
	CFViser(_cfolder,_screen)
{
	compiler=_compiler;
	problem_folder=new char[strlen(_folder)+1];
	strcpy(problem_folder,_folder);
	tests=new STestNode(NULL,-1);
	time_limit=0;
	space_limit=0;
	filter_file=NULL;
	solution_file=NULL;
	grammar_file=NULL;
};

const char* CProblem::get_folder(void)
{
	return (const char*)problem_folder;
};

bool CProblem::filter_exist(void)
{
	return (filter_file!=NULL);
};

const char* CProblem::get_filter_file(void)
{
	return (const char*)filter_file;
};

bool CProblem::solution_exist(void)
{
	return (solution_file!=NULL);
};

const char* CProblem::get_solution_file(void)
{
	return (const char*)solution_file;
};

bool CProblem::grammar_exist(void)
{
	return (grammar_file!=NULL);
};

const char* CProblem::get_grammar_file(void)
{
	return (const char*)grammar_file;
};

int CProblem::update_file(STestInfo::SBody& _body,char*& _file_var,const char* _file)
{
	int _res;
	try
	{
		if(_body.exist)
		{
			if (_file_var!=NULL) 
				delete[] _file_var;
			_file_var=new char[strlen(problem_folder)+strlen(_file)+20];
			sprintf(_file_var,"%s%s",problem_folder,_file);
			if((_res=store_data(_body,_file))!=AER_NO_ERROR)
				throw _res;
		};
	}
	catch(int _error)
	{
		return _error;
	};
	return AER_NO_ERROR;
};

int CProblem::update_test(STestInfo& _test_info)
{
	int _res;
	try
	{
		if((_res=update_file(_test_info.filter,filter_file,TST_FILTER_FILE))!=AER_NO_ERROR)
			throw _res;
		if((_res=update_file(_test_info.solution,solution_file,TST_SOLUTION_FILE))!=AER_NO_ERROR)
			throw _res;
		if((_res=update_file(_test_info.grammar,grammar_file,TST_GRAMMAR_FILE))!=AER_NO_ERROR)
			throw _res;

		if(_test_info.solution.exist)
			strcat(solution_file,".exe");
		if(_test_info.filter.exist)
			strcat(filter_file,".exe");
		
		if(_test_info.test_number!=TST_NO_TEST)
		{
			STestNode* _test=tests->find_number(_test_info.test_number);
			if(_test==NULL)
			{
				char* _folder=new char[strlen(problem_folder)+10];
				try
				{
					sprintf(_folder,"%s%lu\\",problem_folder,_test_info.test_number);
					if((_res=folder->make_folder(_folder))!=AER_NO_ERROR)
						throw _res;
					_test=new STestNode(new CTest(_folder,folder,screen),_test_info.test_number);
					tests->add_next(_test);
				}
				catch(int _error)
				{
					delete[] _folder;
					throw _error;
				}
				delete _folder;
			};
			if((_res=_test->test->update_test(_test_info))!=AER_NO_ERROR)
				throw _res;
		};
	}
	catch (int _error)
	{
		return _error;
	};
	return AER_NO_ERROR;
};

int CProblem::update_task(STaskInfo& _task_info)
{
	time_limit=_task_info.time_limit;
	space_limit=_task_info.space_limit;
	return AER_NO_ERROR;
};

int CProblem::test(const char* _solution,CFilterService* _filter)
{
	char _msg[100];
	int _res;
	screen->print("\n\nStart testing solution.");
	STestNode* _test=tests->next;
	while(_test!=NULL)
	{
		sprintf(_msg,"\ntest %02u: ",_test->number);
		screen->print(_msg);
		if((_res=_test->test->test(_solution,this,_filter))!=ERR_NO_ERROR)
		{
			sprintf(_msg,"error (%s)",get_error_string(_res));
			screen->print(_msg);
			screen->print("\n error.");
			return _res+_test->number;
		};
		screen->print("done.");
		_test=_test->next;
	};
	screen->print("\n done.");
	return ERR_NO_ERROR;
};

unsigned long CProblem::get_time(void)
{
	return time_limit;
};

unsigned long CProblem::get_space(void)
{
	return space_limit;
};

int CProblem::store_data(STestInfo::SBody& _body,const char* _file)
{
	int _res;
	char* _buff=new char[2*_body.size];
	char* _filename=new char[strlen(problem_folder)+strlen(_file)+10];
	try
	{
		sprintf(_filename,"%s%s",problem_folder,_file);
		long _len=base64_decode_binary((unsigned char*)_buff,_body.body);
		if((_res=folder->store_file(_filename,_buff,_len))!=AER_NO_ERROR)
			throw _res;
		
		if(_body.compile)
		{
			char* _tempfile=new char[strlen(folder->get_wf())+strlen(_file)+10];
			try
			{
				folder->make_folder(folder->get_wf());		
				sprintf(_tempfile,"%s%s",folder->get_wf(),_file);
				if((_res=folder->copy_file(_tempfile,_filename))!=AER_NO_ERROR)
					throw _res;
				if((_res=compiler->compile(_tempfile,_body.compiler))!=AER_NO_ERROR)
					throw _res;
				if((_res=folder->delete_file(_filename))!=AER_NO_ERROR)
					throw _res;
				strcat(_tempfile,".exe");
				strcat(_filename,".exe");
				if((_res=folder->copy_file(_filename,_tempfile))!=AER_NO_ERROR)
					throw _res;

			}
			catch(int _error)
			{
				delete[] _tempfile;
				throw _error;
			};
		};
	}
	catch(int _error)
	{
		delete[] _buff;
		delete[] _filename;
		return _error;
	}

	delete _buff;
	delete _filename;
	return AER_NO_ERROR;
};

CProblem::~CProblem()
{
	if(filter_exist())
	{
		folder->delete_file(filter_file);
		delete[] filter_file;
	};
	if(solution_exist())
	{
		folder->delete_file(solution_file);
		delete[] solution_file;
	}
	if(grammar_exist())
	{
		folder->delete_file(grammar_file);
		delete[] grammar_file;
	};
	folder->delete_dir(problem_folder);
	delete[] problem_folder;

};