// Test.cpp - implementation file for Ctest class
#include "Test.h"
#include "TestInfo.h"
#include "Define.h"
#include "MyFunc.h"
#include "FilterService.h"
#include "Problem.h"
#include "Exec.h"
#include <string.h>
#include <stdio.h>
#include <windows.h>

CTest::CTest(const char* _folder,CFolder* _cfolder,CInterface* _screen):
	CExec(_cfolder,_screen)
{
	test_folder=new char[strlen(_folder)+1];
	strcpy(test_folder,_folder);
	input_file=output_file=NULL;
	work=false;
};

const char* CTest::get_folder(void)
{
	return (const char*)test_folder;
};

int CTest::update_test(STestInfo& _test_info)
{
	int _res;
	try
	{
		if(_test_info.input.exist)
		{
			input_file=new char[strlen(test_folder)+strlen(TST_INPUT_FILE)+10];
			sprintf(input_file,"%s%s",test_folder,TST_INPUT_FILE);
			if((_res=store_data(_test_info.input,input_file))!=AER_NO_ERROR)
				throw _res;
		};
		if(_test_info.output.exist)
		{
			output_file=new char[strlen(test_folder)+strlen(TST_OUTPUT_FILE)+10];
			sprintf(output_file,"%s%s",test_folder,TST_OUTPUT_FILE);
			if((_res=store_data(_test_info.output,output_file))!=AER_NO_ERROR)
				throw _res;
		};
	}
	catch(int _error)
	{
		return _error;
	};
	return AER_NO_ERROR;
};

int CTest::store_data(STestInfo::SBody& _body,const char* _path)
{	
	int _res;
	char* _buff=new char[2*_body.size];
	try
	{
		long _len=base64_decode_binary((unsigned char*)_buff,_body.body);
		if((_res=folder->store_file(_path,_buff,_len))!=AER_NO_ERROR)
			throw _res;
	}
	catch(int _error)
	{
		delete[] _buff;
		return _error;
	};
	delete[] _buff;
	return AER_NO_ERROR;
};

int CTest::test(const char* _solution,CProblem* _problem,CFilterService* _filter)
{
	int _res=AER_NO_ERROR;
	char* _test_solution=folder->create_path(folder->get_wf(),TST_TSOLUTION_FILE,".exe");
	char* _solution_file=folder->create_path(folder->get_wf(),TST_SOLUTION_FILE,".exe");
	char* _input_file=folder->create_path(folder->get_wf(),TST_INPUT_FILE,NULL);
	char* _output_file=folder->create_path(folder->get_wf(),TST_OUTPUT_FILE,NULL);
	char* _toutput_file=folder->create_path(folder->get_wf(),TST_TOUTPUT_FILE,NULL);

	try
	{
		if((_res=folder->make_folder(folder->get_wf()))!=AER_NO_ERROR)
			throw _res;
		if((_res=folder->copy_file(_test_solution,_solution))!=AER_NO_ERROR)
			throw _res;
		if((_res=folder->copy_file(_input_file,input_file))!=AER_NO_ERROR)
			throw _res;
		if((_res=execute(_test_solution,"",folder->get_wf(),_problem->get_time(),_problem->get_space()))!=AER_NO_ERROR)
			throw _res;
		if(!folder->file_exists(_output_file))
			throw ERR_WRONG_OUTPUT;
		if((_res=folder->rename_file(_toutput_file,_output_file))!=AER_NO_ERROR)
			throw _res;
		if(_problem->solution_exist())
		{
			if((_res=folder->copy_file(_solution_file,_problem->get_solution_file()))!=AER_NO_ERROR)
				throw _res;
			if((_res=execute(_solution_file,"",folder->get_wf(),_problem->get_time(),_problem->get_space()))!=AER_NO_ERROR)
				throw _res;
			if((_res=folder->delete_file(_solution_file))!=AER_NO_ERROR)
				throw _res;
		}
		else
		{
			if((_res=folder->copy_file(_output_file,output_file))!=AER_NO_ERROR)
				throw _res;
		};
		if((_res=_filter->std_filter1(_toutput_file))!=AER_NO_ERROR)
			throw _res;
		if((_res=_filter->std_filter1(_output_file))!=AER_NO_ERROR)
			throw _res;
		if(_problem->grammar_exist())
		{
			if((_res=_filter->filter2(_toutput_file,_problem->get_grammar_file()))!=AER_NO_ERROR)
				throw _res;
		}
		else
		{
			if((_res=_filter->std_filter2(_toutput_file))!=AER_NO_ERROR)
				throw _res;
		};
		if(_problem->filter_exist())
		{
			if((_res=_filter->filter3(_toutput_file,_input_file,_problem->get_filter_file()))!=AER_NO_ERROR)
				throw _res;
		}
		else
		{
			if((_res=_filter->std_filter3(_toutput_file,_output_file))!=AER_NO_ERROR)
				throw _res;
		};
	}
	catch(int _error)
	{
		_res=_error;	
	};

	folder->delete_path(_test_solution);
	folder->delete_path(_solution_file);
	folder->delete_path(_input_file);
	folder->delete_path(_output_file);
	folder->delete_path(_toutput_file);
	return _res;
};

CTest::~CTest()
{
	if(input_file!=NULL)
	{
		folder->delete_file(input_file);
		delete[] input_file;
	};
	if(output_file!=NULL)
	{
		folder->delete_file(output_file);
		delete[] output_file;
	};
	folder->delete_dir(test_folder);
	delete[] test_folder;
};