// TestService.cpp - implementation for CTestService class
#include "TestService.h"
#include "CompilerService.h"
#include "TaskInfo.h"
#include "Define.h"
#include "MyFunc.h"
#include <string.h>
#include <stdio.h>

// ------------- TaskNode ---------------

CTestService::STaskNode::STaskNode(CProblem* _problem,int _number)
{
	problem=_problem;
	number=_number;
	next=NULL;
};

void CTestService::STaskNode::add_next(STaskNode* _add)
{
	if(next==NULL)
	{
		next=_add;
	}
	else
	{
		if(_add->number>next->number)
			next->add_next(_add);
		else
		{
			_add->next=next;
			next=_add;
		};
	};
};

CTestService::STaskNode* CTestService::STaskNode::find_number(int _number)
{
	if(number==_number)
		return this;
	if(next!=NULL)
		return next->find_number(_number);
	else
		return NULL;
};

CTestService::STaskNode::~STaskNode()
{
	delete problem;
	delete next;
};

// ------------- TestService ---------------

CTestService::CTestService(CFolder* _folder,CInterface* _screen,CCompilerService* _compiler):
	CFViser(_folder,_screen)
{
	compiler=_compiler;
	tasks=new STaskNode(NULL,-1);
	filter=new CFilterService(_folder,_screen);
};

int CTestService::init(char *_buff)
{
	return AER_NO_ERROR;	// no initialization neded
};

int CTestService::update_test(STestInfo& _test_info)
{
	int _res=AER_NO_ERROR;
	STaskNode* _task=tasks->find_number(_test_info.task_number);
	if(_task==NULL)
	{
		char* _folder=new char[strlen(folder->get_tf())+10];
		try
		{
			sprintf(_folder,"%s\\%lu\\",folder->get_tf(),_test_info.task_number);
			if((_res=folder->make_folder(_folder))!=AER_NO_ERROR)
				throw _res;
			_task=new STaskNode(new CProblem(_folder,folder,screen,compiler),_test_info.task_number);
			tasks->add_next(_task);
		}
		catch(int _error)
		{
			delete _folder;
			return _res;
		}
		delete _folder;
	};
	return _task->problem->update_test(_test_info);
};

int CTestService::clear(int _tasknumber)
{
	int _res;
	STaskNode* _task=tasks->find_number(_tasknumber);
	if(_task!=NULL)
	{
		STaskNode* _prev=tasks;
		while(_prev->next!=_task)
			_prev=_prev->next;
		_prev->next=_task->next;
		_task->next=NULL;
		delete _task;
	};
	return AER_NO_ERROR;
};

int CTestService::update_task(STaskInfo& _task_info)
{
	int _res;
	STaskNode* _task=tasks->find_number(_task_info.number);
	if(_task==NULL)
	{
		char* _folder=new char[strlen(folder->get_tf())+10];
		try
		{
			sprintf(_folder,"%s%lu\\",folder->get_tf(),_task_info.number);
			if((_res=folder->make_folder(_folder))!=AER_NO_ERROR)
				throw _res;
			_task=new STaskNode(new CProblem(_folder,folder,screen,compiler),_task_info.number);
			tasks->add_next(_task);
		}
		catch(int _error)
		{
			delete _folder;
			return _res;
		}
		delete _folder;
	};
	return _task->problem->update_task(_task_info);
};

int CTestService::test(const char* _solution,const int _task_number)
{
	char _msg[100];
	
	STaskNode* _task=tasks->find_number(_task_number);
	if(_task==NULL)
	{
		screen->print("error");
		screen->print("\nreceived task number is incorrect. solution ignored");
		return AER_WRONG_TASK;
	};
	return _task->problem->test(_solution,filter);
};

CTestService::~CTestService()
{
	delete tasks;
	delete filter;
};