// Tester.cpp - implementation for CTester class
#include "Tester.h"
#include "Define.h"
#include "CompilerService.h"
#include "TestService.h"
#include "Interface.h"
#include "TaskInfo.h"
#include "TestInfo.h"
#include "Tasks.h"
#include "MyFunc.h"
#include "Solution.h"
#include <windows.h>
#include <stdio.h>

CTester::CTester(CInterface* _screen)
{
	screen=_screen;
	folder=new CFolder(_screen);
	compiler=new CCompilerService(folder,_screen);
	tester=new CTestService(folder,_screen,compiler);
	tasks=new CTasks();
	screen->link(tasks);
	tasks->link(screen);
	tasks->clear();
	handlerMap=new t_handlers();
	formHandlerMap(*handlerMap);
	handler=new Handler(handlerMap, defaultHandler, "OLYMPIA", 1, 0); 
	socket=new stdSocket();
	protocol=new ClientProtocol(socket,handler);
};

void CTester::close(void)
{
	socket->Close();
};

int CTester::tester_init(const char* _filename)
{
	char _msg[100];
	int _res=AER_NO_ERROR;
	HANDLE _ini_file=CreateFile(_filename,
								GENERIC_READ,
								FILE_SHARE_READ,
								NULL,
								OPEN_EXISTING,
								FILE_ATTRIBUTE_NORMAL,
								NULL);
	if(_ini_file==INVALID_HANDLE_VALUE) return IER_CANT_OPEN_FILE;

	unsigned long _file_size=GetFileSize(_ini_file,NULL);
	unsigned long _read_size;
	char* _buff=new char[_file_size+1];

	try
	{
		if(ReadFile(_ini_file,(void*)_buff,_file_size,&_read_size,NULL)==0)
		{
			CloseHandle(_ini_file);
			throw IER_CANT_READ_FILE;
		}
		_buff[_file_size]='\0';

		if(CloseHandle(_ini_file)==0)
			throw IER_CANT_CLOSE_FILE;

		if((_res=screen->initialization(_buff))!=AER_NO_ERROR)
			throw _res;

		sprintf(_msg,"This is tester version 0.74 (last update %s %s)",__TIME__,__DATE__);
		screen->print(_msg);

		screen->print("\n\nLoading network settings...");
		if((_res=this->initialization(_buff))!=AER_NO_ERROR)
			throw _res;
		if(!socket->Created())
			throw IER_SOCKET_NOT_CREATED;

		screen->print("\n\nLoading system folders...");
		if((_res=folder->initialization(_buff))!=AER_NO_ERROR)
			throw _res;
		screen->print("\n\nInitialization internal compilers...");
		if((_res=compiler->initialization(_buff))!=AER_NO_ERROR)
			throw _res;
		screen->print("\n\nInitialization tester subsystem...");
		if((_res=tester->initialization(_buff))!=AER_NO_ERROR)
			throw _res;

		screen->print("\n\nInitialization program complite.");
	}
	catch(int _error)
	{
		_res=_error;
	};
	delete[] _buff;
	return _res;
};

int CTester::connect(void)
{
	if((socket->Connect(host_name,port))==false)
		return RER_CONNETION_FAILED;
	socket->makeNonBlocking();
	return AER_NO_ERROR;
};

void CTester::handle(void)
{
	protocol->Handle();
};

int CTester::init(char* _buff)
{
	char _host[MAX_HOST_LENGTH];
	char _msg[100];
	char _old=' ';
	int _res=AER_NO_ERROR;

	char* _begin=strstr(_buff,SOCKET_CHAPTER);
	if(_begin==NULL)
		return IER_NO_INFORMATION;
	char* _end=strstr(_begin,DOUBLE_NEW_LINE);
	if(_end)
	{
		_old=*_end;
		*_end='\0';
	};

	try
	{
		screen->print("\n\nLoad socket settings...");
		screen->print("\nset hostname to ");
		if(get_string(_buff,SCK_HOST,_host)==0)
			throw IER_WRONG_INI_FILE;
		host_name=new char[strlen(_host)+1];
		strcpy(host_name,_host);
		sprintf(_msg,"\"%s\"",host_name);
		screen->print(_msg);

		screen->print("\nset port to ");
		if(get_uint(_buff,SCK_PORT,&port)==0)
			throw IER_WRONG_INI_FILE;
		sprintf(_msg,"\"%u\"",port);
		screen->print(_msg);
	}
	catch(int _error)
	{
		delete host_name;
		host_name=NULL;
		_res=_error;
	};

	if(_end)
		*_end=_old;

	return _res;
};

int CTester::update_test(STestInfo& _test_info)
{
	return tester->update_test(_test_info);
};

int CTester::update_task(STaskInfo& _task_info)
{
	int _res=tester->update_task(_task_info);
	if(_task_info.number!=TST_NO_TASK)
		tasks->update_task(_task_info.number,_task_info.name);
	return _res;
};

const char* CTester::get_compilers(void)
{
	return compiler->get_compilers();
};

int CTester::test(SSolution& _solution)
{
	int _res;
	char _msg[100];
	char* _test_solution=new char[strlen(folder->get_wf())+strlen(TST_SOLUTION_FILE)+10];
	char* _received_solution=new char[strlen(folder->get_sf())+strlen(TST_SOLUTION_FILE)+10];
	char* _text=new char[_solution.length];
	screen->print("\n\n\nReceive new solution...");
	sprintf(_msg,"\ncompiler: %s   task number: %u",_solution.compilername,_solution.task);
	try
	{
		int _len=base64_decode_binary((unsigned char*)_text,_solution.body);
		sprintf(_received_solution,"%s%s",folder->get_sf(),TST_SOLUTION_FILE);
		if((_res=folder->store_file(_received_solution,_text,_len))!=AER_NO_ERROR)
			throw _res;
		if((*tasks)[_solution.task-1]==NULL)
			throw AER_WRONG_TASK;
		(*tasks)[_solution.task-1]->inc_total();
		if((_res=folder->make_folder(folder->get_wf()))!=AER_NO_ERROR)
			throw _res;
		//Received solution must be already in source folder
		sprintf(_test_solution,"%s%s",folder->get_wf(),TST_SOLUTION_FILE);
		if((_res=folder->copy_file(_test_solution,_received_solution))!=AER_NO_ERROR)
			throw _res;
		if((_res=compiler->compile(_test_solution,_solution.compilername))!=AER_NO_ERROR)
			throw _res;
		strcat(_test_solution,".exe");
		strcat(_received_solution,".exe");
		if((_res=folder->copy_file(_received_solution,_test_solution))!=AER_NO_ERROR)
			throw _res;
		if((_res=tester->test(_received_solution,_solution.task))!=AER_NO_ERROR)
			throw _res;
		(*tasks)[_solution.task-1]->inc_accept();
	}
	catch(int _error)
	{
		if((*tasks)[_solution.task-1]!=NULL)
			(*tasks)[_solution.task-1]->inc_refuse();
		_res=_error;
	};

	delete[] _test_solution;
	return _res;
};

void CTester::Send(RRData& _out)
{
	protocol->Generate(_out);
};

int CTester::clear(int _tasknumber)
{
	return tester->clear(_tasknumber);
};

CTester::~CTester()
{
	delete handlerMap;
	delete handler;
	delete socket;
	delete protocol;
	delete compiler;
	delete tester;
	delete tasks;
	delete folder;
};