//TesterHandler.cpp - implementation of CTesterHandler class
#include "TesterHandler.h"
#include "Console.h"
#include "Tester.h"
#include "TaskInfo.h"
#include "MyFunc.h"
#include "RRData.h"
#include "Define.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

using namespace std;

CTesterHandler::CTesterHandler()
{
	int _res;
	screen=new CConsole();
	tester=new CTester(screen);
	if((_res=tester->init(INI_FILE_NAME))!=AER_NO_ERROR)
	{
		screen->print("\n\n\nCorrect ini file not found. Execution will not continue.\n\n");
	};
	error_code=AER_NO_ERROR;
};

void CTesterHandler::generate_LOGIN(RRData& _out,const char* _compilers)
{
	generate_protocol(_out.protocol);
	_out.command.name="LOGIN";
	_out.command.parameter="tester";
	_out.headers.clear();
	_out.headers.insert(RRData::t_header("Compilers",_compilers));
	_out.body="";
};

void CTesterHandler::generate_LOGOUT(RRData& _out)
{
	generate_protocol(_out.protocol);
	_out.command.name="LOGOUT";
	_out.command.parameter="";
	_out.headers.clear();
	_out.body="";
};

void CTesterHandler::generate_GTT(RRData& _out,const char* _tasks)
{
	generate_protocol(_out.protocol);
	_out.command.name="GTT";
	_out.command.parameter="";
	_out.headers.clear();
	if(_tasks!=NULL)
		_out.headers.insert(RRData::t_header("Tasks",_tasks));
	_out.body="";
};

void CTesterHandler::generate_READY(RRData& _out)
{
	generate_protocol(_out.protocol);
	_out.command.name="READY";
	_out.command.parameter="";
	_out.headers.clear();
	_out.body="";
};

void CTesterHandler::generate_DONE(RRData& _out,int _result,const char* _message,unsigned int _test)
{
	char _str_result[10],_str_test[10];
	sprintf(_str_result,"%d",_result);
	sprintf(_str_test,"%u",_test);
	generate_protocol(_out.protocol);
	_out.command.name="DONE";
	_out.command.parameter="";
	_out.headers.clear();
	_out.headers.insert(RRData::t_header("Result",_str_result));
	_out.headers.insert(RRData::t_header("Message",_message));
	_out.headers.insert(RRData::t_header("Test-Number",_str_test));
	_out.body="";
};

void CTesterHandler::receive_102(RRData& _in)
{
	
};

void CTesterHandler::receive_200(RRData& _in,RRData& _out)
{
	generate_GTT(_out,NULL);
};

void CTesterHandler::receive_203(RRData& _in,RRData& _out)
{
	STaskInfo* _task_info;
	int _task_number,i,j;
	int _blocks_number,_content_length;
	int _res;

	try
	{
		if(_in.headers.find("Tasks-Number")!=_in.headers.end())
		{
			string _str=_in.headers.find("Tasks-Number")->second;
			_task_number=atoi(_str.c_str());
			_task_info=new STaskInfo[_task_number];
			for(i=0;i<_task_number;i++)
				_task_info[i].number=i+1;
		}
		else
		{
			string _str=_in.headers.find("Tasks")->second;
			_task_number=number_of_param(_str.c_str());
			_task_info=new STaskInfo[_task_number];
			for(i=0;i<_task_number;i++)
				_task_info[i].number=int_param(_str.c_str(),i);
		};
		
		string _sname=_in.headers.find("Tasks-Name")->second;
		string _time=_in.headers.find("Time-Limits")->second;
		string _space=_in.headers.find("Space-Limits")->second;

		for(i=0;i<_task_number;i++)
		{
			_task_info[i].name=m_str_param(_sname.c_str(),_task_info[i].number,_task_number);
			_task_info[i].time_limit=m_long_param(_time.c_str(),_task_info[i].number,_task_number);
			_task_info[i].space_limit=m_long_param(_space.c_str(),_task_info[i].number,_task_number);
			if((_res=tester->update_task(_task_info[i]))!=AER_NO_ERROR)
				throw _res;
		};

		for(i=0;i<_task_number;i++)
			delete _task_info[i].name;
		delete[] _task_info;

		string _block=_in.headers.find("Blocks-Number")->second;
		_blocks_number=atoi(_block.c_str());
		string _length=_in.headers.find("Content-Length")->second;
		_content_length=atoi(_length.c_str());

		unsigned long _pos=0;
		for(i=0;i<_blocks_number;i++)
			read_block(_in.body,_pos);

		generate_READY(_out);
	}
	catch(int _error)
	{
		error_code=_error;
	};
};

void CTesterHandler::receive_301(RRData& _in,RRData& _out)
{
	SSolution _solution;
	long _content_length;
	
	string _stask=_in.headers.find("Task")->second;
	_solution.task=atoi(_stask.c_str());
	
	string _scompiler=_in.headers.find("Compiler")->second;
	_solution.compilername=new char[strlen(_scompiler.c_str())];
	strcpy(_solution.compilername,_scompiler.c_str());

	string _slength=_in.headers.find("Content_Length")->second;
	_content_length=atol(_slength.c_str());

	_solution.body=new char[_content_length];
	unsigned long _pos=0;
	read_body(_in.body,_pos,_content_length,_solution.body);
	_solution.length=_content_length;

	int _res=tester->test(_solution);

	generate_DONE(_out,0,"done",0);
};

void CTesterHandler::receive_300(RRData& _in,RRData& _out)
{
	
	if(_in.headers.find("Tasks")==_in.headers.end())
		generate_GTT(_out,NULL);
	else
	{
		string _tasks=_in.headers.find("Tasks")->second;
		generate_GTT(_out,_tasks.c_str());
	};
};

void CTesterHandler::receive_204(RRData& _in,RRData& _out)
{
	//do nothing
};

bool CTesterHandler::string_prefix(const string& _str,const unsigned long _pos,const char* _string)
{
	long i;
	for(i=0;_str[_pos+i]==_string[i] && _string[i]!='\0';i++);
	return (_string[i]=='\0');
};

void CTesterHandler::find_STR(const string& _str,unsigned long& _pos,const char* _string)
{
	_pos=_str.find(_string,_pos);
};

void CTesterHandler::skip_STR(const string& _str,unsigned long& _pos,const char* _string)
{
	unsigned long _next=_str.find(_string,_pos);
	if(_next!=_pos) return;
	_pos+=strlen(_string);
	skip_STR(_str,_pos,_string);
};

void CTesterHandler::skip_string_prefix(const string& _str,unsigned long& _pos, const char* _string)
{
	long i;
	for(i=0;_str[_pos+i]==_string[i] && _string[i]!='\0';i++);
	_pos+=i;
};

void CTesterHandler::read_skip_number(const string& _str,unsigned long& _pos,long& _number)
{
	char* _numb;
	long i;
	for(i=0;isdigit(_str[_pos+i]);i++);
	_numb=new char[i+1];
	for(i=0;isdigit(_str[_pos+i]);i++)
		_numb[i]=_str[_pos+i];
	_numb[i]='\0';
	_pos+=i;
	_number=atol(_numb);
	delete[] _numb;
};

void CTesterHandler::read_skip_string(const string& _str,unsigned long& _pos,char* _buff)
{
	unsigned long _last_pos=_pos;
	long i;
	find_STR(_str,_last_pos,HDR_CRLF);
	for(i=0;_pos+i<_last_pos;i++)
		_buff[i]=_str[_pos+i];
	_buff[i]='\0';
	_pos+=i;
};

void CTesterHandler::read_id_value(const string& _str,unsigned long& _pos,const char* _id,long& _number)
{
	skip_string_prefix(_str,_pos,_id);
	read_skip_number(_str,_pos,_number);
	skip_STR(_str,_pos,HDR_SPACE);
};

void CTesterHandler::read_string_value(const string& _str,unsigned long& _pos,const char* _id,char* _buff)
{
	skip_string_prefix(_str,_pos,_id);
	read_skip_string(_str,_pos,_buff);
	skip_STR(_str,_pos,HDR_SPACE);
};

void CTesterHandler::skip_prefix(const string& _str,unsigned long& _pos,const char* _prefix)
{
	skip_string_prefix(_str,_pos,HDR_TEST_ID);
	skip_STR(_str,_pos,HDR_SPACE);
};

void CTesterHandler::read_body(const string& _str,unsigned long& _pos,const unsigned long _length,char* _buff)
{
	long i;
	for(i=0;i<_length;i++)
	{
		if(string_prefix(_str,_pos,HDR_CRLF))
			skip_STR(_str,_pos,HDR_CRLF);
		_buff[i++]=_str[_pos++];
	};
	find_STR(_str,_pos,HDR_CRLF);
};

void CTesterHandler::read_test_task_block(const string& _str,unsigned long& _pos,STestInfo& _test_info,STestInfo::SBody& _body)
{
	long _length;
	read_id_value(_str,_pos,"Task:",_test_info.task_number);
	read_id_value(_str,_pos,"Test:",_test_info.test_number);
	read_id_value(_str,_pos,"Length:",_length);
	
	_body.body=new char[_length];
	_body.size=_length;
	_body.exist=true;
	read_body(_str,_pos,_length,_body.body);
};

void CTesterHandler::read_task_block(const string& _str,unsigned long& _pos,STestInfo& _test_info,STestInfo::SBody& _body)
{
	long _length;
	read_id_value(_str,_pos,"Task:",_test_info.task_number);
	read_id_value(_str,_pos,"Length:",_length);
	
	_body.body=new char[_length];
	_body.size=_length;
	_body.exist=true;
	read_body(_str,_pos,_length,_body.body);
};

void CTesterHandler::read_task_compiled_block(const string& _str,unsigned long& _pos,STestInfo& _test_info,STestInfo::SBody& _body)
{
	long _length;
	read_id_value(_str,_pos,"Task:",_test_info.task_number);
	read_id_value(_str,_pos,"Length:",_length);
	_body.compile=true;
	_body.compiler=new char[10];
	read_string_value(_str,_pos,"CompilerId:",_body.compiler);
	
	_body.body=new char[_length];
	_body.size=_length;
	_body.exist=true;
	read_body(_str,_pos,_length,_body.body);
}

void CTesterHandler::read_block(const string& _str,unsigned long& _pos)
{
	long _length,i;
	STestInfo _test_info;
	skip_STR(_str,_pos,HDR_CRLF);
	if(string_prefix(_str,_pos,HDR_TEST_ID))
	{
		skip_prefix(_str,_pos,HDR_TEST_ID);
		read_test_task_block(_str,_pos,_test_info,_test_info.input);
		tester->update_test(_test_info);
		return;
	};
	if(string_prefix(_str,_pos,HDR_OUTPUT_ID))
	{
		skip_prefix(_str,_pos,HDR_OUTPUT_ID);
		read_test_task_block(_str,_pos,_test_info,_test_info.output);
		tester->update_test(_test_info);
		return;
	};
	if(string_prefix(_str,_pos,HDR_FILTER_ID))
	{
		skip_prefix(_str,_pos,HDR_FILTER_ID);
		read_task_compiled_block(_str,_pos,_test_info,_test_info.filter);
		tester->update_test(_test_info);
		return;
	};
	if(string_prefix(_str,_pos,HDR_GRAMMAR_ID))
	{
		skip_prefix(_str,_pos,HDR_GRAMMAR_ID);
		read_task_block(_str,_pos,_test_info,_test_info.grammar);
		tester->update_test(_test_info);
		return;
	};
	if(string_prefix(_str,_pos,HDR_SOLUTION_ID))
	{
		skip_prefix(_str,_pos,HDR_SOLUTION_ID);
		read_task_compiled_block(_str,_pos,_test_info,_test_info.solution);
		tester->update_test(_test_info);
		return;
	};
};

int CTesterHandler::int_param(const char* _string,int _nparam)
{
	char _number[10];
	str_param(_string,_nparam,_number);
	return atoi(_number);
};

void CTesterHandler::str_param(const char* _string,int _nparam,char* _buff)
{
	int i,j=0;
	for(i=0;j<_nparam;i++)
		if(_string[i]==';') j++;
	for(j=0;_string[j+i]!=';' && _string[j+i]!='\0';j++)
		_buff[j]=_string[j+i];
	_buff[j]='\0';
};

char* CTesterHandler::m_str_param(const char* _string,int _first,int _all)
{
	char _str[100];
	int i,j,_find;
	for(i=0;i<_all;i++)
	{
		str_param(_string,i,_str);
		for(j=0;_str[j]!='-';j++);
		_str[j]='\0';
		_find=atoi(_str);
		if(_find==_first)
		{
			_str[j++]='-';
			char* _res=new char[strlen(_str+j)+1];
			strcpy(_res,_str+j);
			return _res;
		};
	};
	return NULL;
};

long CTesterHandler::m_long_param(const char* _string, int _first,int _all)
{
	char* _str=m_str_param(_string,_first,_all);
	long _res=atol(_str);
	delete _str;
	return _res;
};

int CTesterHandler::number_of_param(const char* _string)
{
	int i,_number=0;
	for(i=0;i<strlen(_string);i++)
		if(_string[i]==';') _number++;
	return _number+1;
};

void CTesterHandler::generate_protocol(RRData::t_protocol& _protocol)
{
	_protocol.name="OLYMPIA";
	_protocol.version.minor=0;
	_protocol.version.major=1;
};

CTesterHandler::~CTesterHandler()
{
	delete screen;
	delete tester;
};