#include "handler.h"
#include "RRParser.h"
#include "rrdata.h"
#include <iostream>

using std::cout;
using std::endl;

Handler::Handler(t_handlers* _handlers, t_handlerFunction _defaultHandler, string _protocolName, int _major, int _minor, Handler* _base){
	handlers = _handlers;
	base = _base;
	protocol.name = _protocolName;
	protocol.version.major = _major;
	protocol.version.minor = _minor;
	defaultHandler = _defaultHandler;
}

Handler::t_handlerType Handler::getCommandType(string command){
	t_handlers::iterator p = handlers->find(command);
	
	if (p == handlers->end())
		return nocommand;

	return getType((p->second).first);
}

void Handler::Handle(RRData& in, RRData& out){
	// if command line is too long
	if (in.command.name.size() >= MAX_LINE_SIZE){
		(*defaultHandler)(in, out);
		return;
	}

	// if name of the protocol is different
	if (stricmp(in.protocol.name.c_str(), protocol.name.c_str()) != 0){
		(*defaultHandler)(in, out);
		return;
	}

	// if protocol version is higher than we have
	int inMaj = in.protocol.version.major,
		inMin = in.protocol.version.minor,
		ourMaj = protocol.version.major,
		ourMin = protocol.version.minor;
	if ((inMaj > ourMaj) || (inMaj == ourMaj) && (inMin > ourMin)){
		(*defaultHandler)(in, out);
		return;
	}

	// if number of headers is very large
	if (in.headers.size() >= MAX_HEADERS_COUNT){
		(*defaultHandler)(in, out);
		return;
	}

	// looking for handler
	t_handlers::iterator p = handlers->find(in.command.name);
	
	// if nothing has been found
	if (p == handlers->end()){
		(*defaultHandler)(in, out);
		return;
	}

	// if there should be body but there is no Content-Length header
	if (getType((p->second).first) == withbody){
		if (in.headers.find("Content-Length") == in.headers.end()){
			in.headers.insert(RRData::t_header("Content-Length", "-1"));
			(*defaultHandler)(in, out);
			return;
		}
	}


	// we've found something. Let's look closer
	if (inMaj == ourMaj && inMin == ourMin){
		(*((p->second).second))(in, out);
	} else {
		switch (getState((p->second).first)){
			case deprecated:
				(*((p->second).second))(in, out);
				return;
			default:
				if (base != NULL){
					base->Handle(in, out);
					return;
				} else {
					(*defaultHandler)(in, out);
				}
		}
	}
}