#include "handlers.h"
#include "Tester.h"
#include "TaskInfo.h"
#include "Console.h"
#include "MyFunc.h"
#include "RRData.h"
#include "Define.h"
#include "Parse.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

extern CConsole screen;
extern bool done;
extern CTester con_tester;

void generate_protocol(RRData::t_protocol& _protocol)
{
	_protocol.name="OLYMPIA";
	_protocol.version.minor=0;
	_protocol.version.major=1;
};

void defaultHandler(const RRData& in, RRData& _out){
	// check command line length
	if (in.command.name.size() >= MAX_LINE_SIZE){
		generate_DONE(_out,DONE_SERVER_ERROR,"Command length to big");
		con_tester.Send(_out);
		return;
	}

	// check protocol name
	if (in.protocol.name.compare("OLYMPIA") != 0){
		generate_DONE(_out,DONE_SERVER_ERROR,"Incorrect protocol name");
		con_tester.Send(_out);
		return;
	}

	// check protocol version
	if ((in.protocol.version.major != 1) || (in.protocol.version.minor != 0)){
		generate_DONE(_out,DONE_SERVER_ERROR,"Protocol version not supported");
		con_tester.Send(_out);
		return;
	}

	// check number of headers
	if (in.headers.size() >= MAX_HEADERS_COUNT){
		generate_DONE(_out,DONE_SERVER_ERROR,"To many headers");
		con_tester.Send(_out);
		return;
	}

	// check Content-Length existence
	if ((in.headers.find("Content-Length") != in.headers.end()) && (in.headers.find("Content-Length")->second.compare("-1") == 0)){
		generate_DONE(_out,DONE_SERVER_ERROR,"Content length requred");
		con_tester.Send(_out);
		return;
	}

	generate_DONE(_out,DONE_SERVER_ERROR,"Unknown error - bad answer");
	con_tester.Send(_out);
};

void res301(const RRData& _in,RRData& _out)
{
	int _error=receive_301(_in);
	if(_error==RCR_WRONG_ANSWER)
		generate_DONE(_out,DONE_SERVER_ERROR,get_error_string(_error));
	else
	{
		switch(_error/ERR_VALUE)
		{
		case ERR_NO_ERROR:
			if(_error==ERR_NO_ERROR)
				generate_DONE(_out,DONE_NO_ERROR);
			else
				generate_DONE(_out,DONE_TESTER_ERROR,get_error_string(_error));
			break;
		case ERR_COMPILER_NO:
			generate_DONE(_out,DONE_COMPILE_ERROR);
			break;
		case ERR_WRONG_ANSWER_NO:
			generate_DONE(_out,DONE_WRONG_ANSWER_ERROR,NULL,_error%1000);
			break;
		case ERR_TIME_OUT_NO:
			generate_DONE(_out,DONE_TIME_OUT_ERROR,NULL,_error%1000);
			break;
		case ERR_RUNTIME_NO:
			generate_DONE(_out,DONE_RUN_TIME_ERROR,NULL,_error%1000);
			break;
		case ERR_WRONG_OUTPUT_NO:
			generate_DONE(_out,DONE_FORMAT_ERROR,NULL,_error%1000);
			break;
		case ERR_SECURITY_NO:
			generate_DONE(_out,DONE_SECURITY_ERROR,NULL,_error%1000);
			break;
		default:
			generate_DONE(_out,DONE_TESTER_ERROR,get_error_string(_error));
		};
	}
	con_tester.Send(_out);
};

int receive_301(const RRData& _in)
{
	SSolution _solution;
	long _content_length;
	int _res=RCR_NO_ERROR;
	
	try
	{
		if(_in.headers.find("Task")==_in.headers.end())
			throw RCR_WRONG_ANSWER;
		string _stask=_in.headers.find("Task")->second;
		_solution.task=atoi(_stask.c_str());
		
		if(_in.headers.find("Compiler")==_in.headers.end())
			throw RCR_WRONG_ANSWER;
		string _scompiler=_in.headers.find("Compiler")->second;
		_solution.compilername=new char[strlen(_scompiler.c_str())+2];
		strcpy(_solution.compilername,_scompiler.c_str());

		if(_in.headers.find("Content-Length")==_in.headers.end())
			throw RCR_WRONG_ANSWER;
		string _slength=_in.headers.find("Content-Length")->second;
		_content_length=atol(_slength.c_str());

		_solution.body=new char[_content_length+2];
		unsigned long _pos=0;
		read_body(_in.body,_pos,_content_length,_solution.body);
		_solution.length=_content_length;

		_res=con_tester.test(_solution);
	}
	catch(int _error)
	{
		_res=_error;
	};
	return _res;
};

void res500(const RRData& _in,RRData&)
{
	try
	{
		if(_in.headers.find("Message")==_in.headers.end())
			throw RCR_WRONG_ANSWER;
		string _msg=_in.headers.find("Message")->second;
		screen.print("\n\nERROR: internal server error:");
		screen.print(_msg.c_str());
	}
	catch(int)
	{
		screen.print("\n\nERROR: internal server error.");
	};
	done=true;
};

void res400(const RRData& ,RRData&)
{
	screen.print("\n\nERROR: error 400");
	done=true;
};

void res300(const RRData& _in,RRData& _out)
{
	if(_in.headers.find("Tasks")==_in.headers.end())
		generate_GTT(_out,NULL);
	else
	{
		string _tasks=_in.headers.find("Tasks")->second;
		generate_GTT(_out,_tasks.c_str());
	};
	con_tester.Send(_out);
};

void res401(const RRData& ,RRData&)
{
	screen.print("\n\nERROR: error 401");
	done=true;
};

void res406(const RRData& ,RRData&)
{
	screen.print("\n\nERROR: error 406");
	done=true;
};

void res200(const RRData& ,RRData& _out)
{
	generate_GTT(_out,NULL);
	con_tester.Send(_out);
};

void res204(const RRData& ,RRData& _out)
{
	generate_READY(_out);
	con_tester.Send(_out);
};

void res102(const RRData& ,RRData&)
{
};

void res203(const RRData& _in,RRData& _out)
{
	int _error=receive_203(_in);
	if(_error==RCR_NO_ERROR)
		generate_READY(_out);
	else
		if(_error==RCR_WRONG_ANSWER)
			generate_DONE(_out,DONE_SERVER_ERROR,get_error_string(_error));
		else
			generate_DONE(_out,DONE_TESTER_ERROR,get_error_string(_error));
	con_tester.Send(_out);
};

int receive_203(const RRData& _in)
{
	STaskInfo* _task_info=NULL;
	int _task_number=0,i;
	int _res=RCR_NO_ERROR;

	try
	{
		if(_in.headers.find("Tasks-Number")!=_in.headers.end())
		{
			string _str=_in.headers.find("Tasks-Number")->second;
			_task_number=atoi(_str.c_str());
			_task_info=new STaskInfo[_task_number];
			for(i=0;i<_task_number;i++)
				_task_info[i].number=i+1;
		}
		else
		{
			if(_in.headers.find("Tasks")==_in.headers.end())
				throw RCR_WRONG_ANSWER;
			string _str=_in.headers.find("Tasks")->second;
			_task_number=number_of_param(_str.c_str());
			_task_info=new STaskInfo[_task_number];
			for(i=0;i<_task_number;i++)
				_task_info[i].number=int_param(_str.c_str(),i);
		};

		for(i=0;i<_task_number;i++)
			if((_res=con_tester.clear(_task_info[i].number))!=AER_NO_ERROR)
				throw _res;
		
		if(_in.headers.find("Tasks-Names")==_in.headers.end())
			throw RCR_WRONG_ANSWER;
		string _sname=_in.headers.find("Tasks-Names")->second;

		if(_in.headers.find("Time-Limits")==_in.headers.end())
			throw RCR_WRONG_ANSWER;
		string _time=_in.headers.find("Time-Limits")->second;

		if (_in.headers.find("Space-Limits")==_in.headers.end())
			throw RCR_WRONG_ANSWER;
		string _space=_in.headers.find("Space-Limits")->second;

		for(i=0;i<_task_number;i++)
		{
			_task_info[i].name=m_str_param(_sname.c_str(),_task_info[i].number,_task_number);
			_task_info[i].time_limit=1000*m_long_param(_time.c_str(),_task_info[i].number,_task_number);
			_task_info[i].space_limit=m_long_param(_space.c_str(),_task_info[i].number,_task_number);
			if((_res=con_tester.update_task(_task_info[i]))!=AER_NO_ERROR)
				throw _res;
		};

		for(i=0;i<_task_number;i++)
			delete _task_info[i].name;
		delete[] _task_info;
		_task_info=NULL;

		if(_in.headers.find("Blocks-Number")==_in.headers.end())
			throw RCR_WRONG_ANSWER;
		string _block=_in.headers.find("Blocks-Number")->second;
		long _blocks_number=atol(_block.c_str());
		
		if (_in.headers.find("Content-Length")==_in.headers.end())
			throw RCR_WRONG_ANSWER;
		string _length=_in.headers.find("Content-Length")->second;
		long _content_length=atol(_length.c_str());

		unsigned long _pos=0;
		string _str=_in.body;
		STestInfo _test_info;

		for(i=0;i<_blocks_number;i++)
		{
			skip_STR(_in.body,_pos,HDR_CRLF);
			if(string_prefix(_in.body,_pos,HDR_TEST_ID))
			{
				skip_prefix(_in.body,_pos,HDR_TEST_ID);
				skip_STR(_in.body,_pos,HDR_CRLF);
				read_test_task_block(_in.body,_pos,_test_info,_test_info.input);
				if((_res=con_tester.update_test(_test_info))!=AER_NO_ERROR)
					throw _res;
				_test_info.clear();
				continue;
			};
			if(string_prefix(_in.body,_pos,HDR_OUTPUT_ID))
			{
				skip_prefix(_in.body,_pos,HDR_OUTPUT_ID);
				skip_STR(_in.body,_pos,HDR_CRLF);
				read_test_task_block(_in.body,_pos,_test_info,_test_info.output);
				if((_res=con_tester.update_test(_test_info))!=AER_NO_ERROR)
					throw _res;
				_test_info.clear();
				continue;
			};
			if(string_prefix(_in.body,_pos,HDR_FILTER_ID))
			{
				skip_prefix(_in.body,_pos,HDR_FILTER_ID);
				skip_STR(_in.body,_pos,HDR_CRLF);
				read_task_compiled_block(_in.body,_pos,_test_info,_test_info.filter);
				if((_res=con_tester.update_test(_test_info))!=AER_NO_ERROR)
					throw _res;
				_test_info.clear();
				continue;
			};
			if(string_prefix(_in.body,_pos,HDR_GRAMMAR_ID))
			{
				skip_prefix(_in.body,_pos,HDR_GRAMMAR_ID);
				skip_STR(_in.body,_pos,HDR_CRLF);
				read_task_block(_in.body,_pos,_test_info,_test_info.grammar);
				if((_res=con_tester.update_test(_test_info))!=AER_NO_ERROR)
					throw _res;
				_test_info.clear();
				continue;
			};
			if(string_prefix(_in.body,_pos,HDR_SOLUTION_ID))
			{
				skip_prefix(_in.body,_pos,HDR_SOLUTION_ID);
				skip_STR(_in.body,_pos,HDR_CRLF);
				read_task_compiled_block(_in.body,_pos,_test_info,_test_info.solution);
				if((_res=con_tester.update_test(_test_info))!=AER_NO_ERROR)
					throw _res;
				_test_info.clear();
				continue;
			};
		};
		if(_pos!=(unsigned long)_content_length)
			throw RCR_WRONG_ANSWER;
	}
	catch(int _error)
	{
		if(_task_info!=NULL)
		{
			for(i=0;i<_task_number;i++)
				delete _task_info[i].name;
			delete[] _task_info;
		};
		_res=_error;
	};
	return _res;
};

void generate_GTT(RRData& _out,const char* _tasks)
{
	generate_protocol(_out.protocol);
	_out.command.name="GTT";
	_out.command.parameter="";
	_out.headers.clear();
	if(_tasks!=NULL)
		_out.headers.insert(RRData::t_header("Tasks",_tasks));
	_out.body="";
};

void generate_READY(RRData& _out)
{
	generate_protocol(_out.protocol);
	_out.command.name="READY";
	_out.command.parameter="";
	_out.headers.clear();
	_out.body="";
};

void generate_DONE(RRData& _out,int _result,const char* _message,unsigned int _test)
{
	char _str_result[10];
	sprintf(_str_result,"%d",_result);

	generate_protocol(_out.protocol);
	_out.command.name="DONE";
	_out.command.parameter="";
	_out.headers.clear();
	_out.headers.insert(RRData::t_header("Result",_str_result));
	if(_message!=NULL)
		_out.headers.insert(RRData::t_header("Message",_message));
	if(_test!=0)
	{
		char _str_test[10];
		sprintf(_str_test,"%d",_test);
		_out.headers.insert(RRData::t_header("Test-Number",_str_test));
	}
	_out.body="";
};


void generate_LOGIN(RRData& _out,const char* _compilers)
{
	generate_protocol(_out.protocol);
	_out.command.name="LOGIN";
	_out.command.parameter="tester";
	_out.headers.clear();
	_out.headers.insert(RRData::t_header("Compilers",_compilers));
	_out.body="";

}

void formHandlerMap(t_handlers& handlersMap){
	handlersMap.insert(pair<string, t_handler>("203", t_handler(MAKE_HANDLER_TYPE(Handler::recent, Handler::withbody), *res203)));
	handlersMap.insert(pair<string, t_handler>("200", t_handler(MAKE_HANDLER_TYPE(Handler::recent, Handler::withoutbody), *res200)));
	handlersMap.insert(pair<string, t_handler>("301", t_handler(MAKE_HANDLER_TYPE(Handler::recent, Handler::withbody), *res301)));
	handlersMap.insert(pair<string, t_handler>("102", t_handler(MAKE_HANDLER_TYPE(Handler::recent, Handler::withoutbody), *res102)));
	handlersMap.insert(pair<string, t_handler>("204", t_handler(MAKE_HANDLER_TYPE(Handler::recent, Handler::withoutbody), *res204)));
	handlersMap.insert(pair<string, t_handler>("500", t_handler(MAKE_HANDLER_TYPE(Handler::recent, Handler::withoutbody), *res500)));
	handlersMap.insert(pair<string, t_handler>("400", t_handler(MAKE_HANDLER_TYPE(Handler::recent, Handler::withoutbody), *res400)));
	handlersMap.insert(pair<string, t_handler>("401", t_handler(MAKE_HANDLER_TYPE(Handler::recent, Handler::withoutbody), *res401)));
	handlersMap.insert(pair<string, t_handler>("406", t_handler(MAKE_HANDLER_TYPE(Handler::recent, Handler::withoutbody), *res406)));
	handlersMap.insert(pair<string, t_handler>("300", t_handler(MAKE_HANDLER_TYPE(Handler::recent, Handler::withoutbody), *res300)));
}
